use crate::codec::ron::RonCodec;
use crate::components::header::Header;
use crate::models;
use crate::models::HoldPosition;
use crate::models::HoldRole;
use leptos::prelude::*;
use leptos::reactive::graph::ReactiveNode;
use serde::Deserialize;
use serde::Serialize;

#[component]
pub fn Wall() -> impl leptos::IntoView {
    let load = async move {
        // TODO: What to do about this unwrap?
        load_initial_data().await.unwrap()
    };

    leptos::view! {
        <div class="min-w-screen min-h-screen bg-slate-900">
            <div class="container mx-auto">
                <Header />
                <Await future=load let:data>
                    <Ready data=data.to_owned() />
                </Await>
            </div>
        </div>
    }
}

#[component]
fn Ready(data: InitialData) -> impl leptos::IntoView {
    let mut hold_positions = vec![];
    for row in 0..(data.wall.rows) {
        for col in 0..(data.wall.cols) {
            hold_positions.push(HoldPosition { row, col });
        }
    }

    let (current_problem, current_problem_writer) = signal(None::<models::Problem>);
    let problem_fetcher = LocalResource::new(move || async move {
        leptos::logging::log!("Loading random problem");
        let problem = get_random_problem().await.expect("cannot get random problem");
        current_problem_writer.set(Some(problem.into_inner()));
    });

    let mut cells = vec![];
    for &hold_position in &hold_positions {
        let role = move || current_problem.get().and_then(|problem| problem.holds.get(&hold_position).copied());
        let role = Signal::derive(role);

        let cell = view! { <Hold role /> };
        cells.push(cell);
    }

    let grid_classes = format!("grid grid-rows-{} grid-cols-{} gap-4", data.wall.rows, data.wall.cols);

    view! {
        <div class=move || { grid_classes.clone() }>{cells}</div>
        <button on:click=move |_| problem_fetcher.mark_dirty()>"Random problem"</button>
    }
}

#[component]
fn Hold(#[prop(into)] role: Signal<Option<HoldRole>>) -> impl leptos::IntoView {
    let class = move || {
        let role_classes = match role.get() {
            Some(HoldRole::Start) => Some("outline outline-offset-2 outline-green-500"),
            Some(HoldRole::Normal) => Some("outline outline-offset-2 outline-blue-500"),
            Some(HoldRole::Zone) => Some("outline outline-offset-2 outline-amber-500"),
            Some(HoldRole::End) => Some("outline outline-offset-2 outline-red-500"),
            None => Some("brightness-50"),
        };
        let mut s = "bg-indigo-100 aspect-square rounded".to_string();
        if let Some(c) = role_classes {
            s.push(' ');
            s.push_str(c);
        }
        s
    };

    view! { <div class=class></div> }
}

#[derive(Serialize, Deserialize, Clone)]
pub struct InitialData {
    wall: models::Wall,
}

#[server]
async fn load_initial_data() -> Result<InitialData, ServerFnError> {
    use crate::server::state::State;

    let state = expect_context::<State>();

    let wall = state.persistent.with(|s| s.wall.clone()).await;
    Ok(InitialData { wall })
}

#[server]
async fn get_random_problem() -> Result<RonCodec<models::Problem>, ServerFnError> {
    use crate::server::state::State;
    use rand::seq::IteratorRandom;

    let state = expect_context::<State>();

    let problem = state
        .persistent
        .with(|s| {
            let problems = &s.problems.problems;
            let rng = &mut rand::thread_rng();
            problems.iter().choose(rng).unwrap().clone()
        })
        .await;

    tracing::debug!("Returning randomized problem: {problem:?}");

    Ok(RonCodec::new(problem))
}
